// 현재 터미날 창의 가로 세로 길이
int 	screen_size_lines;
int 	screen_size_cols;

// 현재 디렉토리, 오리지날, 목표를 저장
char 	current_path[STRING_MAX];
char 	target_path[STRING_MAX];
char 	original_path[STRING_MAX];

// 이주현 : 현재 디렉토리의 모든 path를 저장하는 구조체
typedef struct {
	int filecount;
	char **path;
	char **statpath;
} filepath_t;

typedef struct {
	char *key;
	char *p_key;
	char *name;
	char *desc;
	char *desc2;
	char *ins;
} menuitem_t;

typedef struct {
	int *n_submenu;
	menuitem_t **submenu;
} menu_t;

typedef struct _PANEL_DATA {
	int hide;       /* TRUE if panel is hidden */
}PANEL_DATA; 

typedef struct {
	int shortkey;
	char *flag;
	char *ins;
} shortkey_t;
