// 공용 변수 라이브러리
//

#define STRING_MAX	1024
#define ARRAY_MAX	255
#define INS_MAX		255

#define FI_MODE		0
#define DI_MODE 	1
#define VI_MODE		2
#define TT_MODE		3
#define TS_MODE		4
#define LG_MODE		5
#define QU_MODE		999	

#define	FALSE		0
#define TRUE		1

#define MY_KEYS (KEY_MAX + 1)

// 이재승
#define COL_OFFSET 4	//디렉토리 하위로 들어갈때의 간격

#define START_COLS 1	//트리가 생성될 윈도우의 시작위치
#define START_ROWS 3


#define MSG_WIN_SIZE_COLS 40	//메시지 윈도우의 크기
#define MSG_WIN_SIZE_ROWS 10



//dirctory struct
typedef struct file_line *file_line_pointer;
typedef struct file_line{
	int line;				//line num  
	char* data;
	file_line_pointer pre;
	file_line_pointer next;
}file_line; 

